@@ -1,3 +1,35 @@
+/*
+ ***** BEGIN LICENSE BLOCK *****
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
+ *
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
+ * 
+ * Copyright (C) 2008-2011 Laurent Gautier
+ * 
+ * Portions created by Alexander Belopolsky are 
+ * Copyright (C) 2006 Alexander Belopolsky.
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the MPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the MPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 #include <Python.h>
 #include <Rdefines.h>
@@ -52,7 +84,8 @@ sexp_strides(SEXP sexp, Py_intptr_t *strides, Py_ssize_t itemsize,
 
 static int
 VectorSexp_getbuffer(PyObject *obj, Py_buffer *view, int flags)
-{  
+{
+
   if (view == NULL) {
     return 0;
   }
@@ -131,41 +164,22 @@ VectorSexp_getbuffer(PyObject *obj, Py_buffer *view, int flags)
 }
 #endif
 
+#if (PY_VERSION_HEX < 0x03010000)
 static Py_ssize_t
 VectorSexp_getsegcount(PySexpObject *self, Py_ssize_t *lenp)
 {
-  printf("getsegcount\n");
-  SEXP sexp = RPY_SEXP(self);
 
-  switch (TYPEOF(sexp)) {
-  case REALSXP:
-    *lenp = GET_LENGTH(sexp) * sizeof(double);
-    break;
-  case INTSXP:
-    *lenp = GET_LENGTH(sexp) * sizeof(int);
-    break;
-  case LGLSXP:
-    *lenp = GET_LENGTH(sexp) * sizeof(int);
-    break;
-  case CPLXSXP:
-    *lenp = GET_LENGTH(sexp) * sizeof(Rcomplex);
-    break;
-  case RAWSXP:
-    *lenp = GET_LENGTH(sexp) * 1;
-    break;
-  default:
-    PyErr_Format(PyExc_ValueError, "Buffer for this type not yet supported.");
-    *lenp = 0;
-    return -1;
+  if (lenp == NULL) {
+    return 1;
+  } else {
+    printf("--->\n");
+    return 0;
   }
-
-  return 0;
 }
 
 static Py_ssize_t
 VectorSexp_getreadbuf(PySexpObject *self, Py_ssize_t segment, void **ptrptr)
 {
-  printf("getreadbuf\n");
   if (segment != 0) {
     PyErr_SetString(PyExc_ValueError,
 		    "accessing non-existing data segment");
@@ -217,12 +231,15 @@ VectorSexp_getcharbuf(PySexpObject *self, Py_ssize_t segment, const char **ptrpt
   /*FIXME: introduce a "writeable" flag for SexpVector objects ? */
   return VectorSexp_getreadbuf(self, segment, (void **)ptrptr);
 }
+#endif
 
 static PyBufferProcs VectorSexp_as_buffer = {
+#if (PY_VERSION_HEX < 0x03010000)
         (readbufferproc)VectorSexp_getreadbuf,
         (writebufferproc)VectorSexp_getwritebuf,
         (segcountproc)VectorSexp_getsegcount,
 	(charbufferproc)VectorSexp_getcharbuf,
+#endif
 #if PY_VERSION_HEX >= 0x02060000
 	(getbufferproc)VectorSexp_getbuffer,
 	(releasebufferproc)0,
