@@ -8,4 +8,8 @@ SEXP rpy_findFun(SEXP symbol, SEXP rho);
 SEXP rpy_serialize(SEXP object, SEXP rho);
 SEXP rpy_unserialize(SEXP connection, SEXP rho);
 
+SEXP rpy_list_attr(SEXP sexp);
+
+const char *RPY_R_VERSION_LIST[5] = {R_MAJOR, R_MINOR, R_STATUS, 
+				     R_SVN_REVISION, NULL};
 #endif
