@@ -1,3 +1,33 @@
+/*
+ ***** BEGIN LICENSE BLOCK *****
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
+ *
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
+ * 
+ * Copyright (C) 2008-2011 Laurent Gautier
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the MPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the MPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
+
 #include <Python.h>
 
 #include "r_utils.h"
@@ -48,8 +78,11 @@ static void
 Sexp_dealloc(PySexpObject *self)
 {
   Sexp_clear(self);
-  self->ob_type->tp_free((PyObject*)self);
-
+#if (PY_VERSION_HEX < 0x03010000)
+  (self->ob_type->tp_free)((PyObject*)self);
+#else
+  (Py_TYPE(self)->tp_free)((PyObject *)self);
+#endif
   /* PyObject_Del(self); */
 }
 
@@ -64,10 +97,17 @@ Sexp_repr(PyObject *self)
    *  return NULL;
    *}
    */
+#if (PY_VERSION_HEX < 0x03010000)
   return PyString_FromFormat("<%s - Python:\%p / R:\%p>",
                              self->ob_type->tp_name,
                              self,
-                             sexp);
+                             sexp);  
+#else
+  return PyUnicode_FromFormat("<%s - Python:\%p / R:\%p>",
+			      self->ob_type->tp_name,
+			      self,
+			      sexp);
+#endif
 }
 
 
@@ -78,35 +118,69 @@ Sexp_typeof_get(PyObject *self)
   SEXP sexp = RPY_SEXP(pso);
   if (! sexp) {
     PyErr_Format(PyExc_ValueError, "NULL SEXP.");
-    return NULL;;
+    return NULL;
   }
+#if (PY_VERSION_HEX < 0x03010000)
   return PyInt_FromLong((long)TYPEOF(sexp));
+#else
+  return PyLong_FromLong((long)TYPEOF(sexp));
+#endif
 }
 PyDoc_STRVAR(Sexp_typeof_doc,
              "R internal SEXPREC type.");
 
 
+PyDoc_STRVAR(Sexp_list_attr_doc,
+             "Returns the list of attribute names.");
+PyObject*
+Sexp_list_attr(PyObject *self)
+{
+  SEXP sexp = RPY_SEXP(((PySexpObject*)self));
+  if (! sexp) {
+    PyErr_Format(PyExc_ValueError, "NULL SEXP.");
+    return NULL;
+  }
+  SEXP res_R;
+  PROTECT(res_R = rpy_list_attr(sexp));
+  PyObject *res = (PyObject *)newPySexpObject(res_R, 1);
+  UNPROTECT(1);
+  return res;
+}
+
 static PyObject*
 Sexp_do_slot(PyObject *self, PyObject *name)
 {
   SEXP sexp = RPY_SEXP(((PySexpObject*)self));
   if (! sexp) {
     PyErr_Format(PyExc_ValueError, "NULL SEXP.");
-    return NULL;;
+    return NULL;
   }
+#if (PY_VERSION_HEX < 0x03010000)
   if (! PyString_Check(name)) {
+#else
+    if (! PyUnicode_Check(name)) {
+#endif
     PyErr_SetString(PyExc_TypeError, "The name must be a string.");
     return NULL;
   }
+#if (PY_VERSION_HEX < 0x03010000)
   char *name_str = PyString_AS_STRING(name);
-
+#else
+  PyObject *pybytes = PyUnicode_AsLatin1String(name);
+  char *name_str = PyBytes_AsString(pybytes);
+#endif
   if (! R_has_slot(sexp, install(name_str))) {
     PyErr_SetString(PyExc_LookupError, "The object has no such attribute.");
+#if (PY_VERSION_HEX >= 0x03010000)
+    Py_DECREF(pybytes);
+#endif
     return NULL;
   }
   SEXP res_R = GET_SLOT(sexp, install(name_str));
-
-  PyObject *res = (PyObject *)newPySexpObject(res_R);
+#if (PY_VERSION_HEX >= 0x03010000)
+    Py_DECREF(pybytes);
+#endif
+  PyObject *res = (PyObject *)newPySexpObject(res_R, 1);
   return res;
 }
 PyDoc_STRVAR(Sexp_do_slot_doc,
@@ -165,18 +239,31 @@ Sexp_named_get(PyObject *self)
     return NULL;;
   }
   unsigned int res = NAMED(sexp);
+#if (PY_VERSION_HEX < 0x03010000)
   return PyInt_FromLong((long)res);
+#else
+  return PyLong_FromLong((long)res);
+#endif
 }
 PyDoc_STRVAR(Sexp_named_doc,
 "Integer code for the R object reference-pseudo counting.\n\
 This method corresponds to the macro NAMED.\n\
 See the R-extensions manual for further details.");
 
+#if (PY_VERSION_HEX < 0x02070000)  
 void SexpObject_CObject_destroy(void *cobj)
 {
   SexpObject* sexpobj_ptr = (SexpObject *)cobj;
   SexpObject_clear(sexpobj_ptr);
 }
+#else
+void SexpObject_CObject_destroy(PyObject *rpycapsule)
+{
+  SexpObject *sexpobj_ptr = (SexpObject *)(PyCapsule_GetPointer(rpycapsule,
+								"rpy2.rinterface._C_API_"));
+  SexpObject_clear(sexpobj_ptr);
+}
+#endif
 
 static PyObject*
 Sexp_sexp_get(PyObject *self, void *closure)
@@ -187,23 +274,54 @@ Sexp_sexp_get(PyObject *self, void *closure)
     PyErr_Format(PyExc_ValueError, "NULL SEXP.");
     return NULL;;
   }
+
   
-  RPY_INCREF(rpyobj);
+  RPY_INCREF(rpyobj);  
+#if (PY_VERSION_HEX < 0x02070000)  
+  /*FIXME: memory leak when INCREF ? */
   PyObject *res = PyCObject_FromVoidPtr(rpyobj->sObj, 
                                         SexpObject_CObject_destroy);
+#else
+  PyObject *res = PyCapsule_New((void *)(rpyobj->sObj),
+				"rpy2.rinterface._C_API_",
+				SexpObject_CObject_destroy);
+#endif
+  
   return res;
 }
 
 static int
 Sexp_sexp_set(PyObject *self, PyObject *obj, void *closure)
 {
+
+#if (PY_VERSION_HEX < 0x02070000)  
   if (! PyCObject_Check(obj)) {
     PyErr_SetString(PyExc_TypeError, "The value must be a CObject.");
     return -1;
   }
+#else
+  if (! PyCapsule_CheckExact(obj)) {
+    PyErr_SetString(PyExc_TypeError, "The value must be a Capsule");
+    return -1;
+  }
+#endif
 
   SexpObject *sexpobj_orig = ((PySexpObject*)self)->sObj;
+
+#if (PY_VERSION_HEX < 0x02070000)  
   SexpObject *sexpobj = (SexpObject *)(PyCObject_AsVoidPtr(obj));
+#else
+  SexpObject *sexpobj = (SexpObject *)(PyCapsule_GetPointer(obj,
+							    "rpy2.rinterface._C_API_"));
+#endif
+
+  if (obj == NULL) {
+    PyErr_SetString(PyExc_TypeError, 
+		    "The value must be a CObject or a Capsule of name 'rpy2.rinterface._C_API_'.");
+    return -1;
+  }
+
+
   #ifdef RPY_DEBUG_COBJECT
   printf("Setting %p (count: %i) to %p (count: %i)\n", 
          sexpobj_orig, (int)sexpobj_orig->count,
@@ -236,6 +354,23 @@ Sexp_sexp_set(PyObject *self, PyObject *obj, void *closure)
 PyDoc_STRVAR(Sexp_sexp_doc,
              "Opaque C pointer to the underlying R object");
 
+static PyObject*
+Sexp_rclass_get(PyObject *self)
+{
+  SEXP sexp = RPY_SEXP(((PySexpObject*)self));
+  if (! sexp) {
+    PyErr_Format(PyExc_ValueError, "NULL SEXP.");
+    return NULL;;
+  }
+
+  SEXP res_R = GET_CLASS(sexp);
+  PyObject *res = (PyObject *)newPySexpObject(res_R, 1);
+  return res;
+}
+PyDoc_STRVAR(Sexp_rclass_doc,
+             "R class name");
+
+
 static PyObject*
 Sexp_refcount_get(PyObject *self)
 {
@@ -243,10 +378,13 @@ Sexp_refcount_get(PyObject *self)
 
   if (! RPY_SEXP(rpyobj)) {
     PyErr_Format(PyExc_ValueError, "NULL SEXP.");
-    return NULL;;
+    return NULL;
   }
-  
+#if (PY_VERSION_HEX < 0x03010000)  
   PyObject *res = PyInt_FromLong((long)(rpyobj->sObj->count));
+#else
+  PyObject *res = PyLong_FromLong((long)(rpyobj->sObj->count));
+#endif
   return res;
 }
 PyDoc_STRVAR(Sexp_refcount_doc,
@@ -294,7 +432,7 @@ Sexp_duplicate(PyObject *self, PyObject *kwargs)
     return NULL;;
   }
   PROTECT(sexp_copy = Rf_duplicate(sexp_self));
-  res = (PyObject *) newPySexpObject(sexp_copy);
+  res = (PyObject *) newPySexpObject(sexp_copy, 1);
   UNPROTECT(1);
   return res;
 }
@@ -325,8 +463,13 @@ Sexp___getstate__(PyObject *self)
           /* res = PyByteArray_FromStringAndSize(sexp_ser, len); */
 
   /*FIXME: is this working on 64bit archs ? */
+#if (PY_VERSION_HEX < 0x03010000)  
   res_string = PyString_FromStringAndSize((void *)RAW_POINTER(sexp_ser), 
                                           (Py_ssize_t)LENGTH(sexp_ser));
+#else
+  res_string = PyBytes_FromStringAndSize((void *)RAW_POINTER(sexp_ser), 
+					 (Py_ssize_t)LENGTH(sexp_ser));
+#endif
   UNPROTECT(1);
   return res_string;
 }
@@ -398,7 +541,7 @@ EmbeddedR_unserialize(PyObject* self, PyObject* args)
                  " (expected %i but got %i)", rtype, TYPEOF(raw_sexp));
     return NULL;
   }
-  res = (PyObject*)newPySexpObject(sexp_ser);
+  res = (PyObject*)newPySexpObject(sexp_ser, 1);
   
   UNPROTECT(2);
   embeddedR_freelock();
@@ -447,16 +590,22 @@ PyDoc_STRVAR(Sexp___reduce___doc,
              "Prepare an instance for serialization.");
 
 
-
 static PyMethodDef Sexp_methods[] = {
+  {"list_attrs", (PyCFunction)Sexp_list_attr, METH_NOARGS,
+   Sexp_list_attr_doc},
   {"do_slot", (PyCFunction)Sexp_do_slot, METH_O,
    Sexp_do_slot_doc},
   {"do_slot_assign", (PyCFunction)Sexp_do_slot_assign, METH_VARARGS,
    Sexp_do_slot_assign_doc},
   {"rsame", (PyCFunction)Sexp_rsame, METH_O,
    Sexp_rsame_doc},
+#if (PY_VERSION_HEX < 0x03010000)  
   {"__deepcopy__", (PyCFunction)Sexp_duplicate, METH_KEYWORDS,
    Sexp_duplicate_doc},
+#else
+  {"__deepcopy__", (PyCFunction)Sexp_duplicate, METH_VARARGS | METH_KEYWORDS,
+   Sexp_duplicate_doc},
+#endif
   {"__getstate__", (PyCFunction)Sexp___getstate__, METH_NOARGS,
    Sexp___getstate___doc},
   {"__setstate__", (PyCFunction)Sexp___setstate__, METH_O,
@@ -476,6 +625,10 @@ static PyGetSetDef Sexp_getsets[] = {
    (getter)Sexp_typeof_get,
    (setter)0,
    Sexp_typeof_doc},
+  {"rclass", 
+   (getter)Sexp_rclass_get,
+   (setter)0,
+   Sexp_rclass_doc},
   {"__sexp__",
    (getter)Sexp_sexp_get,
    (setter)Sexp_sexp_set,
@@ -593,8 +746,12 @@ Sexp_init(PyObject *self, PyObject *args, PyObject *kwds)
 static PyTypeObject Sexp_Type = {
         /* The ob_type field must be initialized in the module init function
          * to be portable to Windows without using C++. */
+#if (PY_VERSION_HEX < 0x03010000)
         PyObject_HEAD_INIT(NULL)
         0,                      /*ob_size*/
+#else
+	PyVarObject_HEAD_INIT(NULL, 0)
+#endif
         "rpy2.rinterface.Sexp",      /*tp_name*/
         sizeof(PySexpObject),   /*tp_basicsize*/
         0,                      /*tp_itemsize*/
