@@ -1,7 +1,7 @@
 import unittest
 import copy
 import gc
-import rpy2.rinterface as rinterface
+from rpy2 import rinterface
 
 rinterface.initr()
 
@@ -27,13 +27,20 @@ class SexpTestCase(unittest.TestCase):
 
     def testTypeof_get(self):
         sexp = rinterface.baseenv.get("letters")
-        self.assertEquals(sexp.typeof, rinterface.STRSXP)
+        self.assertEqual(sexp.typeof, rinterface.STRSXP)
         
         sexp = rinterface.baseenv.get("pi")
-        self.assertEquals(sexp.typeof, rinterface.REALSXP)
+        self.assertEqual(sexp.typeof, rinterface.REALSXP)
         
         sexp = rinterface.baseenv.get("plot")
-        self.assertEquals(sexp.typeof, rinterface.CLOSXP)
+        self.assertEqual(sexp.typeof, rinterface.CLOSXP)
+
+    def testList_attrs(self):
+        x = rinterface.IntSexpVector((1,2,3))
+        self.assertEqual(0, len(x.list_attrs()))
+        x.do_slot_assign('a', rinterface.IntSexpVector((33,)))
+        self.assertEqual(1, len(x.list_attrs()))
+        self.assertTrue('a' in x.list_attrs())
 
     def testDo_slot(self):
         data_func = rinterface.baseenv.get("data")
@@ -42,10 +49,10 @@ class SexpTestCase(unittest.TestCase):
         names = sexp.do_slot("names")
         iris_names = ("Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width", "Species")
 
-        self.assertEquals(len(iris_names), len(names))
+        self.assertEqual(len(iris_names), len(names))
 
         for i, n in enumerate(iris_names):
-            self.assertEquals(iris_names[i], names[i])
+            self.assertEqual(iris_names[i], names[i])
 
         self.assertRaises(LookupError, sexp.do_slot, "foo")  
 
@@ -56,15 +63,15 @@ class SexpTestCase(unittest.TestCase):
         iris_names = rinterface.StrSexpVector(['a', 'b', 'c', 'd', 'e'])
         sexp.do_slot_assign("names", iris_names)
         names = [x for x in sexp.do_slot("names")]
-        self.assertEquals(['a', 'b', 'c', 'd', 'e'], names)
+        self.assertEqual(['a', 'b', 'c', 'd', 'e'], names)
 
     def testDo_slot_assign_create(self):
         #test that assigning slots is also creating the slot
         x = rinterface.IntSexpVector([1,2,3])
         x.do_slot_assign("foo", rinterface.StrSexpVector(["bar", ]))
         slot = x.do_slot("foo")
-        self.assertEquals(1, len(slot))
-        self.assertEquals("bar", slot[0])
+        self.assertEqual(1, len(slot))
+        self.assertEqual("bar", slot[0])
 
     def testSexp_rsame_true(self):
         sexp_a = rinterface.baseenv.get("letters")
@@ -84,15 +91,15 @@ class SexpTestCase(unittest.TestCase):
         sexp = rinterface.IntSexpVector([1,2,3])
         cobj = sexp.__sexp__
         sexp = rinterface.IntSexpVector([4,5,6,7])
-        self.assertEquals(4, len(sexp))
+        self.assertEqual(4, len(sexp))
         sexp.__sexp__ = cobj
-        self.assertEquals(3, len(sexp))
+        self.assertEqual(3, len(sexp))
 
     def testSexp_sexp_wrongtypeof(self):
         sexp = rinterface.IntSexpVector([1,2,3])
         cobj = sexp.__sexp__
         sexp = rinterface.StrSexpVector(['a', 'b'])
-        self.assertEquals(2, len(sexp))
+        self.assertEqual(2, len(sexp))
         self.assertRaises(ValueError, sexp.__setattr__, '__sexp__', cobj)
 
 
@@ -107,23 +114,30 @@ class SexpTestCase(unittest.TestCase):
         
     def testSexp_deepcopy(self):
         sexp = rinterface.IntSexpVector([1,2,3])
-        self.assertEquals(0, sexp.named)
+        self.assertEqual(0, sexp.named)
         rinterface.baseenv.get("identity")(sexp)
-        self.assertEquals(2, sexp.named)
+        self.assertEqual(2, sexp.named)
         sexp2 = sexp.__deepcopy__()
-        self.assertEquals(sexp.typeof, sexp2.typeof)
-        self.assertEquals(list(sexp), list(sexp2))
+        self.assertEqual(sexp.typeof, sexp2.typeof)
+        self.assertEqual(list(sexp), list(sexp2))
         self.assertFalse(sexp.rsame(sexp2))
-        self.assertEquals(0, sexp2.named)
+        self.assertEqual(0, sexp2.named)
         # should be the same as above, but just in case:
         sexp3 = copy.deepcopy(sexp)
-        self.assertEquals(sexp.typeof, sexp3.typeof)
-        self.assertEquals(list(sexp), list(sexp3))
+        self.assertEqual(sexp.typeof, sexp3.typeof)
+        self.assertEqual(list(sexp), list(sexp3))
         self.assertFalse(sexp.rsame(sexp3))
-        self.assertEquals(0, sexp3.named)
+        self.assertEqual(0, sexp3.named)
+
+class RNULLTestCase(unittest.TestCase):
+    def testRNULLType_nonzero(self):
+        NULL = rinterface.RNULLType()
+        self.assertFalse(NULL)
+
 
 def suite():
     suite = unittest.TestLoader().loadTestsFromTestCase(SexpTestCase)
+    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(RNULLTestCase))
     return suite
 
 if __name__ == '__main__':
